#!/bin/bash
#Copyright © 2015, eInfochips Ltd., Inc. All rights reserved.

umask u=rwx,g=rwx,o=rx
#set -x

UNDER='\e[4m'
RED='\e[31;1m'
GREEN='\e[32;1m'
YELLOW='\e[33;1m'
BLUE='\e[34;1m'
MAGENTA='\e[35;1m'
CYAN='\e[36;1m'
WHITE='\e[37;1m'
ENDCOLOR='\e[0m'
ITCVER="Relv1.0"
export WORKDIR=`pwd`
export BUILDROOT="$WORKDIR/LA.AF.1.1-02810-8064.0_$ITCVER"
export PATCH_DIR="$WORKDIR/patches-v1.0"
export CAFTAG="LA.AF.1.1-02810-8064.0.xml"
export OUT=${BUILDROOT}/out/target/product/msm8960/
export CPUS=`grep -c processor /proc/cpuinfo`

function download_CAF_CODE() {
# Do repo sanity test
if [ $? -eq 0 ]
then
	echo "Initializing repo from CAF please wait.."
	cd $BUILDROOT
	repo init  --reference=/mnt/4TB-Part-3/Girish/DB600C/Android/apq8064-la-1-66_amss_device_standalone/LINUX/android-fresh-LA.AF.1.1-02810-8064.0 -u git://codeaurora.org/platform/manifest.git -b release -m ${CAFTAG}
	echo "repo syncing from CAF please wait.."
	rm -f ${BUILDROOT}/.repo/manifest.xml
	cp ${WORKDIR}/manifest_updated.xml ${BUILDROOT}/.repo/manifest.xml
	repo sync -c -f -j${CPUS}
	if [ $? -eq 0 ]
	then
		echo -e "$GREEN Downloading done..$ENDCOLOR"
	else
		echo -e "$RED!!!Error Downloading code!!!$ENDCOLOR"
		exit 1
	fi

	# clone 'libjpegturbo' package
	echo -n "Fetching libjpeg_turbo version, Please wait..."
	cd $BUILDROOT/external/chromium_org/third_party
	git clone git://codeaurora.org/quic/la/platform/external/chromium_org/third_party/libjpeg_turbo
	cd $BUILDROOT/external/chromium_org/third_party/libjpeg_turbo
	git checkout LA.AF.1.1_rb1.18
	git checkout -b Relv1.0
	

	# clone 'boringssl/src' package,
	echo -n "Fetching boringssl version, Please wait..."
	cd $BUILDROOT/external/chromium_org/third_party/boringssl
	git clone git://codeaurora.org/quic/la/platform/external/chromium_org/third_party/boringssl/src
	cd $BUILDROOT/external/chromium_org/third_party/boringssl/src
	git checkout LA.AF.1.1_rb1.18
	git checkout -b Relv1.0

	

	cd $BUILDROOT
	echo "Starting branch Relv1.0 please wait.."
	repo start Relv1.0 --all
	if [ $? -eq 0 ]
	then
		echo -e "$GREEN Repo started branch Relv1.0 done..$ENDCOLOR"
	else
		echo -e "$RED!!!Error repo start branch failed!!!$ENDCOLOR"
	fi
	repo checkout Relv1.0
	if [ $? -eq 0 ]
	then
		echo -e "$GREEN Repo checkedout branch Relv1.0 done..$ENDCOLOR"
	else
		echo -e "$RED!!!Error repo checkout branch failed!!!$ENDCOLOR"
	fi

	cd $BUILDROOT
	rm -rf $BUILDROOT/hardware/qcom/sensors
	tar -xvf ../sensors-st-android-hal.tar.gz -C $BUILDROOT/hardware/qcom/
	if [ $? -eq 0 ]
	then
		echo -e "$GREEN Sensor HAL extraction successfull..$ENDCOLOR"
	else
		echo -e "$RED!!!Sensor HAL update fail!!!$ENDCOLOR"
	fi
else
	echo "repo tool problem, make sure you have setup your build environment"
	echo "1) http://source.android.com/source/initializing.html"
	echo "2) http://source.android.com/source/downloading.html (Installing Repo Section Only)"
	exit -1
fi
}
#####  Function to check result
check_result() {
if [ $? -ne 0 ]
then
	echo

	echo -e "$RED FAIL: Current working dir:$(pwd) $ENDCOLOR"
	echo
	exit 1
else
	echo -e "$GREEN DONE! $ENDCOLOR"
fi
}

apply_android_patches()
{
	echo "Applying patches ..."
	if [ ! -e $PATCH_DIR ]
	then
		echo -e "$RED $PATCH_DIR : Not Found $ENDCOLOR"
	fi
	cd $PATCH_DIR
	patch_root_dir="$PATCH_DIR"
	android_patch_list=$(find . -type f -name "*.patch" | sort) &&
	for android_patch in $android_patch_list; do
		android_project=$(dirname $android_patch)
		echo -e "$YELLOW   applying patches on $android_project ... $ENDCOLOR"
		cd $BUILDROOT/$android_project
		if [ $? -ne 0 ]; then
			echo -e "$RED $android_project does not exist in BUILDROOT:$BUILDROOT $ENDCOLOR"
			exit 1
		fi
		git am $patch_root_dir/$android_patch
		check_result
	done
}
######  Function to check whether an application exists
check_program() {
for cmd in "$@"
do
	which ${cmd} > /dev/null 2>&1
	if [ $? -ne 0 ]
	then
		echo
		echo -e "$RED Cannot find command \"${cmd}\" $ENDCOLOR"
		echo
		exit 1
	fi
done
}

#Main Script starts here
# Note: Check necessary program for installation
echo
echo -e "$MAGENTA eInfochips Release Version : $ITCVER $ENDCOLOR"
echo -e "$MAGENTA Workdir                   : $WORKDIR $ENDCOLOR"
echo -e "$MAGENTA Build Root                : $BUILDROOT $ENDCOLOR"
echo -e "$MAGENTA Patch Dir                 : $PATCH_DIR $ENDCOLOR"
echo -e "$MAGENTA Codeaurora TAG            : $CAFTAG $ENDCOLOR"
echo -n "Checking necessary program for installation......"
echo

if [ -e $BUILDROOT ]
then
	cd $BUILDROOT
else
	mkdir $BUILDROOT
	cd $BUILDROOT
fi

#curl https://dl-ssl.google.com/dl/googlesource/git-repo/repo > ${BUILDROOT}/repo
#curl http://commondatastorage.googleapis.com/git-repo-downloads/repo > ${BUILDROOT}/repo

check_program tar repo git java

#1 Download code
download_CAF_CODE
cd $WORKDIR

#2 Apply SD 600eval board patches
apply_android_patches
cd $BUILDROOT

echo -e "$YELLOW   Extracting proprietary image to ${BUILDROOT}/vendor/qcom/ ... $ENDCOLOR"
tar -xvf ../proprietary-Android-LA.AF.1.1-02810-8064.0-SD_600eval-v1.0.tar.gz -C vendor/qcom/

echo -e "$YELLOW   Extracting SD 600eval Documents to ${BUILDROOT}/out/target/product/msm8960/data/ ... $ENDCOLOR"
mkdir -p $BUILDROOT/out/target/product/msm8960/data/
tar -xvf ../SD_600eval_Documents.tar.gz -C $BUILDROOT/out/target/product/msm8960/data/

#mkdir -p ${BUILDROOT}/out/target/product/msm8960/obj/SHARED_LIBRARIES/libsurround_proc_intermediates/
#mkdir -p ${BUILDROOT}/out/target/product/msm8960/obj/SHARED_LIBRARIES/audio.primary.msm8960_intermediates/
#cp ${BUILDROOT}/vendor/qcom/proprietary/prebuilt/target/product/msm8960/obj/SHARED_LIBRARIES/libsurround_proc_intermediates/export_includes out/target/product/msm8960/obj/SHARED_LIBRARIES/libsurround_proc_intermediates/export_includes
#cp ${BUILDROOT}/vendor/qcom/proprietary/prebuilt/target/product/msm8960/obj/SHARED_LIBRARIES/audio.primary.msm8960_intermediates/import_includes out/target/product/msm8960/obj/SHARED_LIBRARIES/audio.primary.msm8960_intermediates/import_includes

echo -e "$YELLOW   Sourcing Android build environment ... $ENDCOLOR"
cd $BUILDROOT
source build/envsetup.sh

echo -e "$YELLOW   Choosing Platform to build ... $ENDCOLOR"
lunch msm8960-userdebug

echo -e "$YELLOW   Starting Build ... $ENDCOLOR"
make -j ${CPUS}

cd $BUILDROOT
echo -e "$YELLOW Note:Below command can be used to build the source"
echo -e "$GREEN cd $BUILDROOT $ENDCOLOR"
echo -e "$GREEN source build/envsetup.sh $ENDCOLOR"
echo -e "$GREEN lunch msm8960-userdebug $ENDCOLOR"
echo -e "$GREEN make -j4 $ENDCOLOR"
