#ifndef __LOWI_CONST_H__
#define __LOWI_CONST_H__

/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

        LOWI Const Interface Header file

GENERAL DESCRIPTION
  This file contains the structure definitions and function prototypes for
  LOWIConst

Copyright (c) 2012 Qualcomm Atheros, Inc.
  All Rights Reserved.
  Qualcomm Atheros Confidential and Proprietary.

Export of this technology or software is regulated by the U.S. Government.
Diversion contrary to U.S. law prohibited.
=============================================================================*/
#include <ctype.h>
#include <inttypes.h>
#include <time.h>
#include <new>
#include <stdint.h>
#include <base_util/vector.h>
#include <inc/lowi_defines.h>

namespace qc_loc_fw
{

#define BSSID_LEN 6
#define SSID_LEN 32
#define MAX_CHAN_INFO_SIZE 16
#define CLIENT_NAME_LEN 128
#define CELL_POWER_NOT_FOUND 0x7F
#define LOWI_COUNTRY_CODE_LEN 2

const char* const LOWI_CLIENT_SUFFIX = "-LOWIClient";
const char* const SERVER_NAME = "LOWI-SERVER";
const char* const NOT_AVAILABLE = "NA";
const int ASYNC_DISCOVERY_REQ_VALIDITY_PERIOD_SEC_MAX = 3600;

// minimum measurement period (in msec) for a wifi node in an ranging periodic request
uint32 const LOWI_MIN_MEAS_PERIOD = 500;
// maximum number of measurement retries attempted by the lowi scheduler on a wifi node
uint8  const MAX_RETRIES_PER_MEAS = 3;
// maximum number of channels allowed in a bgscan bucket specification
uint8 const LOWI_MAX_CHANNELS_GSCAN = 16;
// maximum number of buckets allowed in a bgscan request
uint8 const LOWI_MAX_BUCKETS = 16;
// maximum number of wifi nodes allowed in a hotlist request
uint32 const LOWI_MAX_HOTLIST_APS = 128;
// maximum number of wifi nodes allowed in a significant change request
uint32 const LOWI_MAX_SIGNIFICANT_CHANGE_APS = 64;

} // namespace qc_loc_fw
#endif //#ifndef __LOWI_CONST_H__
