/*============================================================================
   Copyright (c) 2013 Qualcomm Technologies, Inc. All Rights Reserved
   Qualcomm Technologies Proprietary and Confidential.
============================================================================*/
#ifndef __TINTLESS_WRAPPER_H__
#define __TINTLESS_WRAPPER_H__

/*============================================================================
DEFINITIONS and CONSTANTS
============================================================================*/
// API version of this wrapper
#define TINTLESS_WRAPPER_VERSION 1
// Functional changes under this version
#define TINTLESS_WRAPPER_MINOR_VERSION 0

/* Define a tintless library to be used */
#define QC_TINTLESS

/*============================================================================
INCLUDE FILES
============================================================================*/

/*============================================================================
Type DECLARATIONS
============================================================================*/

typedef enum {
  TINTLESS_WRAPPER_SUCCESS               =  0,
  TINTLESS_WRAPPER_NO_MEMORY             = -1,
  TINTLESS_WRAPPER_ERROR                 = -2,
  TINTLESS_WRAPPER_INVALID_STATS         = -3,
  TINTLESS_WRAPPER_BAD_OUTPUT_TABLE      = -4,
  TINTLESS_WRAPPER_LIB_NOT_LOADED        = -5,
  TINTLESS_WRAPPER_LIB_MISMATCH          = -6,
  TINTLESS_WRAPPER_UPDATES_NOT_SUPPORTED = -7,
  TINTLESS_WRAPPER_UNKNOWN_ERROR         = -8
} tintless_wrapper_return_t;

typedef enum {
  UPDATES_STAT_CONFIG,
  UPDATES_AEC,
  UPDATES_AWB,
  UPDATES_AF,
  UPDATES_CAMERA_MODE,
  UPDATES_FLASH_MODE,
  UPDATES_CHROMATIX_PARAMS,
  UPDATES_END
} tintless_wrapper_update_request_t;

#define TINTLESS_WRAPPER_SUPPORTED_UPDATES \
    ((1 << UPDATES_STAT_CONFIG) | \
/*   (1 << UPDATES_AEC) | \
     (1 << UPDATES_AWB) | \
     (1 << UPDATES_AF) | \
     (1 << UPDATES_CAMERA_MODE) | \
     (1 << UPDATES_FLASH_MODE) | \
*/   (1 << UPDATES_CHROMATIX_PARAMS))

/*============================================================================
EXTERNAL STRUCTS - BAYER AND ROLL OFF TABLES
============================================================================*/
#define TINTLESS_MESH_ROLLOFF_SIZE (17 * 13)

// Defines mesh based roll off correction tables
typedef struct
{
/**< Size of Mesh Tables */
  uint16_t table_size;
/**< Red Channel Mesh Table */
  float  r_gain[TINTLESS_MESH_ROLLOFF_SIZE];
  /**< GreenRed Channel Mesh Table */
  float  gr_gain[TINTLESS_MESH_ROLLOFF_SIZE];
/**< GreenBlue Channel Mesh Table */
  float  gb_gain[TINTLESS_MESH_ROLLOFF_SIZE];
/**< Blue Channel Mesh Table */
  float  b_gain[TINTLESS_MESH_ROLLOFF_SIZE];
} mesh_rolloff_array_t;

/*
* bayer_grid_stats_info_t defines the Bayer grid stats information.
*/
typedef struct
{
  // Pointer to an array where each value in the array corresponds to the
  // number pixels in a region of a given channel that are below the
  // threshold saturation value.
  const uint32_t *channel_counts;

  // Pointer to an array where each value in the array corresponds to the sum
  // of the pixels in a region of a given channels that are below the
  // threshold saturation value.
  const uint32_t *channel_sums;

  // Size of each array. All the array sizes are same.
  // The array size will be the total number of regions.
  uint32_t array_length;
} bayer_grid_stats_info_t;

typedef struct {
  uint32_t  camif_win_w;    // width of the camif window
  uint32_t  camif_win_h;    // heigth of the camif window
  uint32_t  stat_elem_w;    // width of one stat element
  uint32_t  stat_elem_h;    // heigth of one stat element
} stats_config_t;

typedef struct {
  unsigned char tint_correction_strength;
} chromatix_params_t;
/*============================================================================
Function DECLARATIONS
============================================================================*/

tintless_wrapper_return_t
vfe_tintless_wrapper_init(void (*pFunc)(unsigned int *));

tintless_wrapper_return_t
vfe_tintless_wrapper_get_version(
   uint32_t *const version);

tintless_wrapper_return_t
vfe_tintless_wrapper_config(const tintless_wrapper_update_request_t type,
                            void *cfg);
tintless_wrapper_return_t
vfe_tintless_wrapper_update(bayer_grid_stats_info_t *pbayer_r,
                            bayer_grid_stats_info_t *pbayer_gr,
                            bayer_grid_stats_info_t *pbayer_gb,
                            bayer_grid_stats_info_t *pbayer_b,
                            mesh_rolloff_array_t  *ptable_current,
                            mesh_rolloff_array_t  *ptable_3a,
                            mesh_rolloff_array_t  *ptable_correction);
tintless_wrapper_return_t vfe_tintless_wrapper_reload_params(void *params);
tintless_wrapper_return_t vfe_tintless_wrapper_deinit();
#endif /* __TINTLESS_WRAPPER_H__ */
