/*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
 *  Copyright (c) 2013 Qualcomm Technologies, Inc.  All Rights Reserved.     *
 *  Qualcomm Technologies Proprietary and Confidential.                      *
 *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*/

#ifndef _EXPORT_EPDSP
#define _EXPORT_EPDSP

#define NO_COMMAND              0
#define COMMAND_OUT_SETGAIN 10
#define COMMAND_OUT_SPEAKER     30
#define COMMAND_OUT_TEMPERATURE 40

#define SPEAKER_ON              1
#define SPEAKER_OFF             0

#define MAX_DUMP_DATA_SIZE 25

/****************************** COMMAND IN *****************************/
//Acoustics
#define COMMAND_AGC_only    0x210
#define COMMAND_Spur_Light  0x220
#define COMMAND_Spur_Remover    0x230
#define COMMAND_Get_Spur_Frequency  0x240
#define COMMAND_IIR_High_Pass   0x250
#define COMMAND_TIMEBASELOCK    0x260
//Pen Interference
#define COMMAND_Pen_Interference_X_length   0x0310
#define COMMAND_Pen_Interference_Y_length   0x0320
//Template related
#define COMMAND_Wiener_filter   0x0410
//Post Processing
#define COMMAND_Motion_Filter   0x0510
#define COMMAND_Smoother    0x0520
#define COMMAND_Top_Linearity   0x0530
#define COMMAND_Bilinear    0x0540
#define COMMAND_Parabolic   0x0550
//Triangulation
#define COMMAND_Large_area_wrap 0x0610
//Temperature
#define COMMAND_SpeakerOnOff    0x0700
//Debug
#define COMMAND_Traces  0xF010
#define COMMAND_Scope_Debugger  0xF020
#define COMMAND_Packet_Logger   0xF030


/****************************** PACKET HEADERS ****************************/
#define  IR_PACKET_TYPE_A  64
#define  IR_PACKET_TYPE_B  65
#define  IR_PACKET_TYPE_C  66

#define  TEMPERATURE_PACKET  0xF4

#define TRACE_ERROR 0
#define TRACE_WARN 1
#define TRACE_INFO 2
#define TRACE_DEBUG 3

typedef struct
{
            unsigned long BaseID;
            unsigned long TerminalID;
}EPSource;


typedef struct
{
    EPSource        Source;
    long X;
    long Y;
    long    Z;
    long    P;
    long    Type;
    long    TiltX;
    long    TiltY;
}EPoint;

typedef struct
{
    long ActiveState;
    long ActivePenNumber;
    long CommandOut[4];
}FeedbackInfo;

typedef struct
{
    long                ReceivedGain;
    long                GainChangeCounter;
    long                RequestedDeltaGain;
    long                RequestedGain;
    long                CalculatedPower;
    long                GainRequestReady;
    long                Reset;
}AGC_struct;

typedef struct
{
    long DumpID;
    long Length;
    long PacketNumber;
    long Data[MAX_DUMP_DATA_SIZE];
}DumpPacket;

typedef long (*WriteDumpCallback)(DumpPacket *Dump, long Length);
typedef void (*TraceCallback)(int trace_level, const char *fmt, ...);

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_MSC_VER)
#if defined(EPDSP_EXPORTS)
#define EPDSP_API __declspec(dllexport)
#else
#define EPDSP_API __declspec(dllimport)
#endif
#elif defined(__GNUC__)
#define EPDSP_API __attribute__((visibility("default")))
#else
#define EPDSP_API
#endif

void EPDSP_API GetDSPVersion (char *OutDSPVersionString, unsigned char *OutDSPVersion);
void EPDSP_API GetAllocationSizes (long *OutPointMaxCountPerPen, long *OutMaxPens, long *OutSizeOfWorkspace);
long EPDSP_API InitDSP(EPoint *InPointBuffer, void *InWorkspace, void *InConfiguration, long ConfigureationLength);
long EPDSP_API GetPoints(long *pPacket, void *InWorkspace, FeedbackInfo *OutFeedback);
void EPDSP_API ReleaseDSP(void *InWorkspace, void *OutConfiguration);
void EPDSP_API AGC(short *inPtr16,long *input512,AGC_struct *pAGC,long ChannelNumber,long EnableEvents);
void EPDSP_API InitAGC(AGC_struct *pAGC);
void EPDSP_API UpdateAGC(AGC_struct *pAGC,short Gain);
long EPDSP_API Command (void *InWorkspace, long *CommandBuffer);
void EPDSP_API InitDumpCallbacks(WriteDumpCallback WriteDumpFunc);
long EPDSP_API ResetDSP(EPoint *InPointBuffer, void *InWorkspace);
long EPDSP_API LoadCoeffs(void *InWorkspace, void *InConfiguration, long InConfigLength);
int EPDSP_API GetPersistentData(void *InWorkspace, long *Buffer, int Length);
void EPDSP_API SetTraceCallback(TraceCallback callback);

#ifdef __cplusplus
}
#endif

//SCRATCH BUFFER
extern char Scratch[1024*6];


#endif //_EXPORT_EPDSP
