/*
 * Copyright (C) 2012 STMicroelectronics
 * Matteo Dameno, Denis Ciocca, Alberto Marinoni - Motion MEMS Product Div.
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "configuration.h"
#if (SENSORS_TEMPERATURE_ENABLE == 1) || (SENSORS_PRESSURE_ENABLE == 1)

#ifndef ANDROID_PRESS_SENSOR_H
#define ANDROID_PRESS_SENSOR_H

#include <stdint.h>
#include <errno.h>
#include <sys/cdefs.h>
#include <sys/types.h>

#include "sensors.h"
#include "SensorBase.h"
#include "InputEventReader.h"

#if PRESS_COMPENSATION_ENABLE == 1
extern "C" {
	#include "pressureCompensationAPI.h"
};

#define PRESS_COMPENSATION_FILE_NAME		"compensation_param"
#endif

/*****************************************************************************/

struct input_event;

class PressTempSensor : public SensorBase {

	enum {
		Pressure = 0,
		Temperature,
		numSensors
	};
	static unsigned int mEnabled;
	static int64_t delayms;
	static int current_fullscale;
	uint32_t mPendingMask;
	InputEventCircularReader mInputReader;
	sensors_event_t mPendingEvents[numSensors];
	bool mHasPendingEvent;

	int setInitialState();

private:
	enum channelid{
		pressChan = 0,
		tempChan
	};

	char device_sysfs_path_prs[PATH_MAX];
	int device_sysfs_path_prs_len;

#if PRESS_COMPENSATION_ENABLE == 1
	struct CalibrationPackage calvalues;
	virtual int initCompensationAlgo(void);
#endif

public:
	PressTempSensor();
	virtual ~PressTempSensor();
	virtual int readEvents(sensors_event_t* data, int count);
	virtual bool hasPendingEvents() const;
	virtual int setDelay(int32_t handle, int64_t ns);
	virtual int setFullScale(int32_t handle, int value);
	virtual int enable(int32_t handle, int enabled, int type);
	virtual int getWhatFromHandle(int32_t handle);
};

#endif  // ANDROID_PRESS_SENSOR_H

#endif /* SENSORS_TEMPERATURE_ENABLE | SENSORS_PRESSURE_ENABLE */
